/* texeqn: TeX equation stripping
 * to compile:   cc texeqn.c -o texeqn */

char *documentation[] = {
" SYNTAX",
"        texeqn [-iw] file1 [file2 .....]",
"     or texeqn [-iw]  < file1 [file2 ....]",
"",
"        Flags:",
"              -i     ignores TeX's and LaTeX's \input files",
"              -w     matching is not checked",
"",
"See the manual page for more details.",
"",
};

/* Author: Kamal Al-Yahya, Stanford University,		11/1/83 */
/* Modified:						6/30/86 */

int	doclength = { sizeof documentation/sizeof documentation[0] };

#include        <stdio.h>
#include        <sys/ioctl.h>
#include        <sgtty.h>
#define	MAXLEN	100000

struct sgttyb ttystat;
extern char *strcpy(), *mktemp();
char scratch_file[100];

int wflag;
int xargc;
char **xargv;

main(argc,argv)
int argc; 
char *argv[];
{
char big[MAXLEN];
FILE *temp,*scr;
register char *cptr;
int piped_in;
int iflag,i;

/* If no arguments, and not in a pipeline, self document */
piped_in = ioctl ((fileno (stdin)), TIOCGETP, &ttystat);
if (argc == 1 && !piped_in)
	{
	for( i=0; i<doclength; i++)
		printf("%s\n",documentation[i]);
	exit (0);
	}

/* process option flags */
xargc = argc;
xargv = argv;
for (xargc--,xargv++; xargc; xargc--,xargv++)
	{
	cptr = *xargv; 
	if( *cptr=='-' )
		{
		while( *(++cptr))
			{
			switch( *cptr )
				{
				case 'i':
					iflag=1;
					break;
				case 'w':
					wflag=1;
					break;
				default:
			     		fprintf(stderr,
					"unknown flag -%c\n",*cptr);
					break;
				}
			}
		}
	}

/* first process pipe input */
if(piped_in)
	{
/* need to buffer; can't seek in pipes */
/* make a temporary and volatile file in /tmp */
	strcpy(scratch_file,"/tmp/texXXXXXX");
	mktemp(scratch_file);
	scr=fopen(scratch_file,"w");
	scrbuf(stdin,scr);
	fclose(scr);
	scr=fopen(scratch_file,"r");
	unlink(scratch_file);
	if (wflag != 1)
		{
		fprintf(stderr,"Checking matching...\n");
		TeXMatch(scr);
		fseek(scr,0,0);
		}
/* either expand or buffer */
	if (iflag != 1)
		{ TeXExpand(scr,big,MAXLEN);	fclose(scr); }
	else
		{ tmpbuf(scr,big);		fclose(scr); }
	if (wflag != 1)
		fprintf(stderr,"Checking matching done\n\n");
	TeXEqn(big);
	fclose(scr);
	}

/* then process input line for arguments and assume they are input files */
xargc = argc;
xargv = argv;
for (xargc--,xargv++; xargc; xargc--,xargv++)
	{
	cptr = *xargv; 
	if( *cptr=='-' ) continue;		/* this is a flag */
	if((temp=fopen(cptr,"r")) != NULL)
		{
		if (wflag != 1)
			{
			fprintf(stderr,"Checking matching...\n");
			fprintf(stderr,"%s:\n",cptr);
			TeXMatch(temp);
			fprintf(stderr,"\n");
			fseek(temp,0,0);
			}
/* either expand or buffer */
		if (iflag != 1)
			{ TeXExpand(temp,big,MAXLEN);	fclose(temp); }
		else
			{ tmpbuf(temp,big);		fclose(temp); }
		if (wflag != 1)
			fprintf(stderr,"Checking matching done\n\n");
		TeXEqn(big);
		fclose(temp);
		}
	else
		fprintf(stderr,"texeqn: Cannot open %s\n",cptr);
	}

}

TeXEqn(buffer)			/* srips TEX equations */
char *buffer;
{
int c,d;
int i,j,be;
while ((c = *buffer++) != '\0')
	{
	if(c == '%')
		{
		while ((c = *buffer++) != '\0')
			if (c == '\n') break;
		}
	if(c == '$')
		{
		if ((d = *buffer++) == '$')
			{
			putc(c,stdout);	putc(d,stdout);
			while ((c = *buffer++) != '\0')
				{
				if(c != '$')   putc(c,stdout);
				else
					{
					buffer++;
					fprintf(stdout,"$$ \n");
					break;
					}
				}
			}
		}
	if(c == '\\')
		{			/* check for LaTeX \begin{equation} */
		if ((c = *buffer++) =='b')
		if ((i=begin_end_buf(buffer,&be)) == 0)
			{
			if (be == 1)
				{
				fprintf(stdout,"\\begin{equation}");
				buffer += 14;
				while ((c = *buffer++) != '\0')
					{
					if (c != '\\')
						putc(c,stdout);
					else	putc('\\',stdout);
					if((i=begin_end_buf(buffer,&be)) > 0)
						{
						for (j=0; j < i; j++)
							{
							c = *buffer++;
							putc(c,stdout);
							}
						}
					else if (be == 2)
					    {
					    fprintf(stdout,"end{equation}\n");
					    break;
					    }
					}
				}
			}
		}
	}
}
